@@ -461,6 +461,7 @@ mc_plot_raster <- function(data, filename=NULL, sensors=NULL, by_hour=TRUE, png_
 #' * `facet = NULL`, all localities and sensors (max 2 physicals, see details) are plotted in single plot
 #' @param color_by_logger If TRUE, the color is assigned by logger to differentiate individual loggers (random colors)
 #' if false, the color is assigned by physical. (default FALSE)
+#' @param tag hilight states with selected tag. (default NULL)
 #' @return ggplot2 object
 #' @examples
 #' tms.plot <- mc_filter(mc_data_example_agg, localities = "A6W79")
@@ -475,23 +476,28 @@ mc_plot_line <- function(data, filename=NULL, sensors=NULL,
                          start_crop=NULL, end_crop=NULL, use_utc=TRUE,
                          localities=NULL,
                          facet="locality",
-                         color_by_logger=FALSE) {
+                         color_by_logger=FALSE,
+                         tag=NULL) {
     data <- mc_filter(data, localities=localities, sensors=sensors)
     if(!is.null(start_crop) || !is.null(end_crop)) {
         data <- mc_prep_crop(data, start_crop, end_crop)
     }
     sensors_table <- .plot_get_sensors_table(data, facet)
     sensors_table <- .plot_add_coeff_to_sensors_table(sensors_table, scale_coeff, facet)
-    data_table <- .plot_reshape_long(data, use_utc=use_utc)
-    change_colors <- .plot_get_change_colors(data, data_table, facet, color_by_logger)
-    data_table <- .plot_line_edit_data_table(data_table, sensors_table, change_colors, facet)
+    data_long <- .plot_reshape_long(data, use_utc=use_utc)
+    plot_settings <- .plot_get_plot_line_settings(data, facet, color_by_logger)
+    data_table <- .plot_line_edit_data_table(data_long, sensors_table, plot_settings, facet)
 
-    plot <- ggplot2::ggplot(data=data_table, ggplot2::aes(x=.data$datetime, y=.data$value_coeff, group=.data$series_name)) +
-            ggplot2::geom_line(ggplot2::aes(color=.data$series_name))
-    if(!change_colors) {
-        plot <- plot + ggplot2::scale_color_manual(values=sensors_table$color)
+    plot <- ggplot2::ggplot(data=data_table, ggplot2::aes(x=.data$datetime, y=.data$value_coeff,
+                                                          group=.data$series, fill=.data$series,
+                                                          colour=.data$series)) +
+            ggplot2::geom_line(ggplot2::aes(color=.data$series))
+    plot <- .plot_line_add_states_if_selected(plot, data, data_long, sensors_table, tag, plot_settings, facet)
+    if(!plot_settings$change_colors) {
+        plot <- .plot_line_set_sensor_colors(plot, data_table, sensors_table)
     } else {
         plot <- plot + ggplot2::scale_color_manual(values=.plot_const_PALETTE)
+        plot <- plot + ggplot2::scale_fill_manual(values=.plot_const_PALETTE)
     }
     plot <- plot + .plot_set_ggplot_line_theme()
     plot <- plot + .plot_line_set_y_axes(sensors_table)
@@ -601,14 +607,18 @@ mc_plot_line <- function(data, filename=NULL, sensors=NULL,
     sensors_table
 }
 
-.plot_line_edit_data_table <- function(data_table, sensors_table, change_colors, facet) {
-    if(change_colors) {
-        data_table$series_name <- paste(data_table$locality_id, data_table$sensor_name)
-    } else {
-        data_table$series_name <- data_table$sensor_name
+.plot_line_edit_data_table <- function(data_table, sensors_table, plot_settings, facet) {
+    if (plot_settings$show_locality && plot_settings$show_logger) {
+        data_table$series <- paste(data_table$locality_id, data_table$logger_name, data_table$sensor_name)
     }
-    if(any(data_table$logger_index != 1)) {
-        data_table$series_name <- paste0(data_table$series_name, " (", data_table$logger_index, ")")
+    else if (plot_settings$show_locality) {
+        data_table$series <- paste(data_table$locality_id, data_table$sensor_name)
+    }
+    else if (plot_settings$show_logger) {
+        data_table$series <- paste(data_table$logger_name, data_table$sensor_name)
+    }
+    else {
+        data_table$series <- data_table$sensor_name
     }
 
     coeff_list <- as.list(sensors_table$coeff)
@@ -619,11 +629,112 @@ mc_plot_line <- function(data, filename=NULL, sensors=NULL,
     {
         join_table <- dplyr::select(sensors_table, "sensor", "physical")
         names(join_table) <- c("sensor_name", "physical")
-        data_table <- dplyr::left_join(data_table, join_table, by=dplyr::join_by("sensor_name"))
+        data_table <- dplyr::left_join(data_table, join_table, by="sensor_name")
     }
     return(data_table)
 }
 
+.plot_line_add_states_if_selected <- function(plot, data, data_long, sensors_table, tag, plot_settings, facet) {
+    if(is.null(tag)) {
+        return(plot)
+    }
+    states_data_long <- .plot_get_states_data(plot, data, data_long, tag)
+    if(nrow(states_data_long) == 0) {
+        return(plot)
+    }
+    is_point_data <- states_data_long$group == "point"
+    states_data_long_area <- dplyr::filter(states_data_long, !is_point_data)
+    states_data_long_point <- dplyr::filter(states_data_long, is_point_data)
+    if(nrow(states_data_long_area) > 0) {
+        states_data_table_area <- .plot_line_edit_data_table(states_data_long_area, sensors_table, plot_settings, facet)
+        plot <- plot + ggplot2::geom_area(data=states_data_table_area, ggplot2::aes(group=.data$group),
+                                          alpha=0.3, position="identity", show.legend = FALSE)
+    }
+    if(nrow(states_data_long_point) > 0) {
+        states_data_table_point <- .plot_line_edit_data_table(states_data_long_point, sensors_table, plot_settings, facet)
+        plot <- plot + ggplot2::geom_point(data=states_data_table_point, show.legend = FALSE, shape=8)
+    }
+    return(plot)
+}
+
+.plot_get_states_data <- function(plot, data, data_long, tag) {
+    states_table <- .plot_line_get_states_table(data, tag)
+    states_table <- dplyr::select(states_table, "locality_id", "logger_name", "sensor_name", "start", "end")
+
+    filter_function <- function(locality_id, logger_name, sensor_name, start, end) {
+        result <- data_long$locality_id == locality_id &
+                  data_long$sensor_name == sensor_name &
+                  data_long$datetime >= start &
+                  data_long$datetime <= end
+        if (!is.na(logger_name)) {
+            result <- result & data_long$logger_name == logger_name
+        }         
+        return(result)
+    }
+
+    conditions <- purrr::pmap(states_table, filter_function)
+
+    table_function <- function(condition, i) {
+        result <- dplyr::filter(data_long, condition)
+        if(nrow(result) == 1) {
+            result$group <- "point"
+        } else {
+            result$group <- stringr::str_glue("area_{i}")
+        }
+        return(result)
+    }
+
+    states_data_long <- purrr::imap_dfr(conditions, table_function)
+
+    return(states_data_long)
+}
+
+.plot_line_get_states_table <- function(data, state_tag) {
+    is_agg <- .common_is_agg_format(data)
+    
+    sensor_function <- function(locality_id, logger_name, sensor) {
+        result <- dplyr::filter(sensor$states, .data$tag == state_tag)
+        result <- dplyr::mutate(result, locality_id=locality_id, logger_name=logger_name, sensor_name=sensor$metadata@name)
+        return(result)
+    }
+
+    logger_function <- function(locality_id, logger) {
+        params <- list(locality_id=locality_id,
+                       logger_name=logger$metadata@name,
+                       sensor=logger$sensors)
+        return(purrr::pmap_dfr(params, sensor_function))
+    }
+
+    locality_function <- function(locality) {
+        if(is_agg) {
+            params <- list(locality_id=locality$metadata@locality_id,
+                           logger_name=NA_character_,
+                           sensor=locality$sensors)
+            result <- purrr::pmap_dfr(params, sensor_function)
+        } else {
+            params <- list(locality_id=locality$metadata@locality_id,
+                           logger=locality$loggers)
+            result <- purrr::pmap_dfr(params, logger_function)
+        }   
+        return(result)   
+    }
+
+    states_table <- purrr::map_dfr(data$localities, locality_function)
+
+    return(states_table)
+}
+
+.plot_line_set_sensor_colors <- function(plot, data_table, sensors_table) {
+    series_table <- dplyr::distinct(data_table, .data$sensor_name, .data$series)
+    temp_color_table <- dplyr::select(sensors_table, "sensor", "color")
+    color_table <- dplyr::left_join(series_table, temp_color_table, by=c("sensor_name"="sensor"))
+    color_values <- color_table$color
+    names(color_values) <- color_table$series
+    plot <- plot + ggplot2::scale_color_manual(values=color_values)
+    plot <- plot + ggplot2::scale_fill_manual(values=color_values)
+    return(plot)
+}
+
 .plot_line_set_y_axes <- function(sensors_table) {
     physical_table <- dplyr::distinct(dplyr::select(sensors_table, "physical", "main_axis", "coeff"))
     sec_axis <- .plot_line_get_sec_y_axis(physical_table)
@@ -693,26 +804,29 @@ mc_plot_line <- function(data, filename=NULL, sensors=NULL,
 .plot_reshape_long <- function(data, use_utc=TRUE) {
     is_raw_format <- .common_is_raw_format(data)
 
-    sensor_function <- function(locality_id, logger_index, sensor_item, datetime) {
+    sensor_function <- function(locality_id, logger_name, sensor_item, datetime) {
         count <- length(datetime)
         tibble::tibble(locality_id=rep(locality_id, count),
-                       logger_index=rep(logger_index, count),
+                       logger_name=rep(logger_name, count),
                        sensor_name=rep(sensor_item$metadata@name, count),
                        datetime=datetime,
                        value=sensor_item$values)
     }
 
-    sensors_item_function <- function(locality_id, logger_index, tz_offset, item) {
+    sensors_item_function <- function(locality_id, tz_offset, item) {
         tz_offset <- if(use_utc) 0 else tz_offset
         datetime <- .calc_get_datetimes_with_offset(item$datetime, tz_offset)
-        tables <- purrr::pmap_dfr(list(locality_id=locality_id, logger_index=logger_index,
+        logger_name <- NA_character_
+        if(is_raw_format) {
+            logger_name <- item$metadata@name
+        }
+        tables <- purrr::pmap_dfr(list(locality_id=locality_id, logger_name=logger_name,
                                        sensor_item=item$sensors, datetime=list(datetime)),
                                   sensor_function)
     }
 
     raw_locality_function <- function(locality) {
         purrr::pmap_dfr(list(locality_id=locality$metadata@locality_id,
-                             logger_index=seq_along(locality$loggers),
                              tz_offset=locality$metadata@tz_offset,
                              item=locality$loggers), sensors_item_function)
     }
@@ -721,19 +835,28 @@ mc_plot_line <- function(data, filename=NULL, sensors=NULL,
         result <- purrr::map_dfr(data$localities, raw_locality_function)
     } else {
         result <- purrr::pmap_dfr(list(locality_id=names(data$localities),
-                                       logger_index=1,
                                        tz_offset=purrr::map(data$localities, ~ .x$metadata@tz_offset),
                                        item=data$localities), sensors_item_function)
     }
     return(result)
 }
 
-.plot_get_change_colors <- function(data, data_table, facet, color_by_logger) {
-    if(any(data_table$logger_index != 1) && color_by_logger) {
-        return(TRUE)
+.plot_get_plot_line_settings <- function(data, facet, color_by_logger) {
+    is_not_locality_facet <- is.null(facet) || facet != .plot_const_FACET_LOCALITY
+    result <- list()
+    result$show_locality <- is_not_locality_facet && length(data$localities) > 1
+    
+    locality_diff_logger_types <- function(locality) {
+        types <- purrr::map_chr(locality$loggers, ~ .x$metadata@type)
+        return(any(duplicated(types)))
     }
-    if(is.null(facet) || facet != .plot_const_FACET_LOCALITY) {
-        return(length(data$localities) > 1)
+
+    if(.common_is_agg_format(data)){
+        result$show_logger <- FALSE
+    } else {
+        result$show_logger <- any(purrr::map_lgl(data$localities, locality_diff_logger_types))
     }
-    return(FALSE)
+
+    result$change_colors <- color_by_logger && (result$show_locality || result$show_logger)
+    return(result)
 }
