@@ -15,8 +15,9 @@
 .prep_const_MESSAGE_RECLEAN <- "MyClim object is already cleaned. Repeated cleaning overwrite cleaning informations."
 .prep_const_MESSAGE_ALREADY_CALIBRATED <- "It is not possible change calibration parameters in calibrated sensor."
 .prep_const_MESSAGE_DATETIME_WRONG_TYPE <- "Type of datetime column must be POSIXct."
-.prep_const_MESSAGE_CROP_DATETIME_LENGTH <- paste0("Start and end datetime can be NULL, ",
-                                                   "single value or vector with same length as localities.")
+.prep_const_MESSAGE_CROP_DATETIME_LENGTH <- "Start or end can be either NULL or single value. Multiple values are not allowed. For advance cropping use crop_table parameter."
+.prep_const_MESSAGE_CROP_TABLE_PARAMS <- "Cropping must be defined either as a parameter or as a table."
+.prep_const_MESSAGE_CROP_TABLE_DUPLICATES <- "Crop table contains duplicated items."
 .prep_const_MESSAGE_VALUES_SAME_TIME <- "In logger {serial_number} are different values of {sensor_name} in same time."
 .prep_const_MESSAGE_STEP_PROBLEM <- "step cannot be detected for logger {logger$metadata@serial_number} - skip"
 .prep_const_MESSAGE_CLEAN_CONFLICT <- "Object not cleaned. The function only tagged (states) measurements with cleaning conflicts."
@@ -39,9 +40,9 @@
 #' The function `mc_prep_clean` can be used in two different ways depending on 
 #' the parameter `resolve_conflicts`. When `resolve_conflicts=TRUE`, the function
 #' performs automatic cleaning and returns a cleaned myClim object. When `resolve_conflicts=FALSE`,
-#' and myClim object contains conflicts, the function returns the original, 
-#' uncleaned object with tags (states) see [mc_states_insert]
-#' highlighting records with duplicated datetime but different measurement values.
+#' and myClim object contains conflicts (rows with identical time, but different measured value), 
+#' the function returns the original, uncleaned object with tags (states) see [mc_states_insert]
+#' highlighting records with duplicated datetime but different measured values.
 #' When there were no conflicts, cleaning is performed in both cases (`resolve_conflicts=TRUE OR FALSE`)
 #' 
 #' Processing the data with `mc_prep_clean` and resolving the conflicts is a mandatory step
@@ -53,8 +54,26 @@
 #' (used time-step is permanently stored in logger metadata [myClim::mc_LoggerMetadata];
 #' or if time-step is not provided by the user (NA),than myClim automatically
 #' detects the time-step from input time series based on the last 100 records.
-#' In case of irregular time series, function returns warning and skip the series.
+#' In case of irregular time series, function returns warning and skip (does not read) the file.
 #'
+#' In cases when the user provides a time-step during data import in `mc_read` functions 
+#' instead of relying on automatic step detection, and the provided step does not correspond 
+#' with the actual records (i.e., the logger records data every 900 seconds but the user 
+#' provides a step of 3600 seconds), the myClim rounding routine consolidates multiple 
+#' records into an identical datetime. The resulting value corresponds to the one closest 
+#' to the provided step (i.e., in an original series like ...9:50, 10:05, 10:20, 10:35, 10:50, 11:05..., 
+#' the new record would be 10:00, and the value will be taken from the original record at 10:05). 
+#' This process generates numerous warnings in `resolve_conflicts=TRUE` and a multitude of tags 
+#' in `resolve_conflicts=FALSE`.
+#' 
+#' The `tolerance` parameter is designed for situations where the logger does not perform optimally,
+#' but the user still needs to extract and analyze the data. In some cases, loggers may record 
+#' multiple rows with identical timestamps but with slightly different microclimate values, 
+#' due to the limitations of sensor resolution and precision.
+#' By using the `tolerance` parameter, myClim will automatically select one of these values 
+#' and resolve the conflict without generating additional warnings. It is strongly recommended 
+#' to set the `tolerance` value based on the sensor's resolution and precision.
+#' 
 #' In case the time-step is regular, but is not nicely rounded, function rounds
 #' the time series to the closest nice time and shifts original data.
 #' E.g., original records in 10 min regular step c(11:58, 12:08, 12:18, 12:28)
@@ -65,33 +84,26 @@
 #' When you have 2h time step and wish to go to the whole hour  
 #' (13:33 -> 14:00, 15:33 -> 16:00) the only way is aggregation - 
 #' use `mc_agg(period="2 hours")` command after data cleaning.
-#' 
-#' In cases when the user provides a time-step during data import in `mc_read` functions 
-#' instead of relying on automatic step detection, and the provided step does not correspond 
-#' with the actual records (i.e., the logger records data every 900 seconds but the user 
-#' provides a step of 3600 seconds), the myClim rounding routine consolidates multiple 
-#' records into an identical datetime. The resulting value corresponds to the one closest 
-#' to the provided step (i.e., in an original series like ...9:50, 10:05, 10:20, 10:35, 10:50, 11:05..., 
-#' the new record would be 10:00, and the value will be taken from the original record at 10:05). 
-#' This process generates numerous warnings in `resolve_conflicts=TRUE` and a multitude of tags 
-#' in `resolve_conflicts=FALSE`.
 #'  
 #' @template param_myClim_object_raw
 #' @param silent if true, then cleaning log table and progress bar is not printed in console (default FALSE), see [myClim::mc_info_clean()]
 #' @param resolve_conflicts by default the object is automatically cleaned and conflict 
 #' measurements with closest original datetime to rounded datetime are selected, see details. (default TRUE)
-#' If FALSE and conflict records exist the function returns the original, uncleaned object with tags (states) "conflict"
+#' If FALSE and conflict records exist the function returns the original, uncleaned object with tags (states) "clean_conflict"
 #' highlighting records with duplicated datetime but different measurement values.When conflict records 
 #' does not exist, object is cleaned in both TRUE and FALSE cases. 
+#' @param tolerance list of tolerance values for each physical unit see [mc_data_physical].
+#' Format is list(unit_name=tolerance_value). If maximal difference of conflict values is lower then tolerance,
+#' conflict is resolved without warning. If NULL, then tolerance is not applied (default NULL) see details. 
 #' @return
 #' * cleaned myClim object in Raw-format (default) `resolve_conflicts=TRUE` or `resolve_conflicts=FALSE` but no conflicts exist 
 #' * cleaning log is by default printed in console, but can be called also later by [myClim::mc_info_clean()]
-#' * non cleaned myClim object in Raw-format with "conflict" tags `resolve_conflicts=FALSE` and conflicts exist
+#' * non cleaned myClim object in Raw-format with "clean_conflict" tags `resolve_conflicts=FALSE` and conflicts exist
 #' 
 #' @export
 #' @examples
 #' cleaned_data <- mc_prep_clean(mc_data_example_raw)
-mc_prep_clean <- function(data, silent=FALSE, resolve_conflicts=TRUE) {
+mc_prep_clean <- function(data, silent=FALSE, resolve_conflicts=TRUE, tolerance=NULL) {
     if(.common_is_agg_format(data)) {
         stop(.prep_const_MESSAGE_CLEAN_AGG)
     }
@@ -100,16 +112,14 @@ mc_prep_clean <- function(data, silent=FALSE, resolve_conflicts=TRUE) {
     }
     clean_env <- new.env()
     clean_env$resolve_conflicts <- resolve_conflicts
+    clean_env$tolerance <- tolerance
     clean_env$states <- tibble::tibble()
-    clean_env$clean_bar <- NULL
     count_table <- mc_info_count(data)
-    if(!silent) {
-        clean_env$clean_bar <- progress::progress_bar$new(format = "clean [:bar] :current/:total loggers",
-                                                          total=count_table$count[count_table$item == "loggers"])
-    }
+    clean_env$clean_bar <- progress::progress_bar$new(format = "clean [:bar] :current/:total loggers",
+                                                      total=count_table$count[count_table$item == "loggers"])
     locality_function <- function(locality) {
-        locality$loggers <- purrr::imap(locality$loggers, ~ .prep_clean_logger(locality$metadat@locality_id,
-                                                                               .x, .y, clean_env))
+        locality$loggers <- purrr::map(locality$loggers, ~ .prep_clean_logger(locality$metadat@locality_id,
+                                                                              .x, clean_env))
         locality
     }
 
@@ -127,7 +137,7 @@ mc_prep_clean <- function(data, silent=FALSE, resolve_conflicts=TRUE) {
     return(cleaned_data)
 }
 
-.prep_clean_logger <- function(locality_id, logger, logger_index, clean_env) {
+.prep_clean_logger <- function(locality_id, logger, clean_env) {
     if(is.na(logger$metadata@step)) {
         logger$clean_info@step <- .prep_detect_step_seconds(logger$datetime)
     } else {
@@ -135,7 +145,7 @@ mc_prep_clean <- function(data, silent=FALSE, resolve_conflicts=TRUE) {
     }
     if(is.na(logger$clean_info@step)) {
         warning(stringr::str_glue(.prep_const_MESSAGE_STEP_PROBLEM))
-        if(!is.null(clean_env$clean_bar)) clean_env$clean_bar$tick()
+        clean_env$clean_bar$tick()
         return(logger)
     }
     rounded_datetime <- .prep_get_rounded_datetime(logger)
@@ -143,9 +153,9 @@ mc_prep_clean <- function(data, silent=FALSE, resolve_conflicts=TRUE) {
     original_datetime <- logger$datetime
     logger$datetime <- rounded_datetime
     logger <- .prep_clean_write_info(logger, rounded)
-    logger <- .prep_clean_edit_series(locality_id, logger, logger_index, original_datetime, clean_env)
+    logger <- .prep_clean_edit_series(locality_id, logger, original_datetime, clean_env)
     logger <- .prep_clean_round_states(logger)
-    if(!is.null(clean_env$clean_bar)) clean_env$clean_bar$tick()
+    clean_env$clean_bar$tick()
     logger
 }
 
@@ -188,7 +198,7 @@ mc_prep_clean <- function(data, silent=FALSE, resolve_conflicts=TRUE) {
     logger
 }
 
-.prep_clean_edit_series <- function(locality_id, logger, logger_index, original_datetime, clean_env) {
+.prep_clean_edit_series <- function(locality_id, logger, original_datetime, clean_env) {
     if(!.prep_clean_was_error_in_logger(logger)){
         return(logger)
     }
@@ -206,8 +216,10 @@ mc_prep_clean <- function(data, silent=FALSE, resolve_conflicts=TRUE) {
         sensor_table$original_diff <- NULL
         if(any(duplicated_datetime))
         {
-            .prep_clean_check_different_values_in_duplicated(locality_id, logger_index, sensor_table,
-                                                             logger$metadata@serial_number, clean_env)
+            sensor_id <- logger$sensors[[sensor_name]]$metadata@sensor_id
+            physical <- myClim::mc_data_sensors[[sensor_id]]@physical
+            .prep_clean_check_different_values_in_duplicated(locality_id, logger$metadata@name, sensor_table,
+                                                             logger$metadata@serial_number, clean_env, physical)
             return(sensor_table[!duplicated_datetime, ])
         }
         return(sensor_table)
@@ -223,33 +235,47 @@ mc_prep_clean <- function(data, silent=FALSE, resolve_conflicts=TRUE) {
     logger
 }
 
-.prep_clean_check_different_values_in_duplicated <- function(locality_id, logger_index, sensor_table, serial_number, clean_env){
+.prep_clean_check_different_values_in_duplicated <- function(locality_id, logger_name, sensor_table, serial_number, clean_env, physical){
     duplicated_rows <- duplicated(sensor_table$datetime) | duplicated(sensor_table$datetime, fromLast = TRUE)
     duplicated_table <- dplyr::filter(sensor_table, duplicated_rows)
-    groupped_duplicated <- dplyr::group_by(duplicated_table, .data$datetime)
-    is_different_function <- function(.x, .y) {
-        result <- !all(diff(.x[[1]]) == 0)
-        return(rep(result, nrow(.x)))
+    duplicated_table <- dplyr::group_by(duplicated_table, .data$datetime)
+    tolerance_value <- NA_real_
+    if(!is.null(clean_env$tolerance) && physical %in% names(clean_env$tolerance)) {
+        tolerance_value <- clean_env$tolerance[[physical]]
     }
-    is_different <- as.logical(purrr::flatten(dplyr::group_map(groupped_duplicated, is_different_function)))
-    if(any(is_different)) {
+    is_different_function <- function(.x, .y) {
+        min_value <- min(.x[[1]], na.rm=TRUE)
+        max_value <- max(.x[[1]], na.rm=TRUE)
+        if(is.na(tolerance_value)) {
+            result <- !dplyr::near(min_value, max_value)
+        } else {
+            result <- !dplyr::near(min_value, max_value, tol = tolerance_value)
+        }
+        return(result)
+    }
+    different_table <- tibble::tibble(
+        datetime=unique(duplicated_table$datetime),
+        is_different=as.logical(dplyr::group_map(duplicated_table, is_different_function)))
+    if(any(different_table$is_different)) {
         sensor_name <- names(sensor_table)[[2]]
         warning(stringr::str_glue(.prep_const_MESSAGE_VALUES_SAME_TIME))
+        sensor_table <- dplyr::left_join(sensor_table, different_table, by="datetime")
+        sensor_table$is_different[is.na(sensor_table$is_different)] <- FALSE
         if(!clean_env$resolve_conflicts) {
-            .prep_add_conflict_states(locality_id, logger_index, sensor_name, sensor_table, is_different, clean_env)
+            .prep_add_conflict_states(locality_id, logger_name, sensor_name, sensor_table, clean_env)
         }
     }
 }
 
-.prep_add_conflict_states <- function(locality_id, logger_index, sensor_name,
-                                      sensor_table, is_different, clean_env) {
-    diff_parts <- rle(is_different)
+.prep_add_conflict_states <- function(locality_id, logger_name, sensor_name,
+                                      sensor_table, clean_env) {
+    diff_parts <- rle(sensor_table$is_different)
     ends <- cumsum(diff_parts$lengths)
     starts <- c(1, ends[-length(ends)] + 1)
     ends <- ends[diff_parts$values]
     starts <- starts[diff_parts$values]
-    states_table <- tibble::tibble(locality_id=locality_id, logger_index=logger_index,
-                                   sensor_name=sensor_name, tag=.model_const_SENSOR_STATE_CONFLICT,
+    states_table <- tibble::tibble(locality_id=locality_id, logger_name=logger_name,
+                                   sensor_name=sensor_name, tag=.model_const_SENSOR_STATE_CLEAN_CONFLICT,
                                    start=sensor_table$datetime[starts], end=sensor_table$datetime[ends],
                                    value=NA_character_)
     clean_env$states <- dplyr::bind_rows(clean_env$states, states_table)
@@ -269,10 +295,10 @@ mc_prep_clean <- function(data, silent=FALSE, resolve_conflicts=TRUE) {
 
 .prep_clean_round_states <- function(logger) {
     step <- logger$clean_info@step
-    start_datetime <- dplyr::first(logger$datetime)
+    start <- dplyr::first(logger$datetime)
 
     sensor_function <- function(sensor) {
-        return(.states_floor_sensor(sensor, start_datetime, step))
+        return(.states_floor_sensor(sensor, start, step))
     }
 
     logger$sensors <- purrr::map(logger$sensors, sensor_function)
@@ -282,10 +308,14 @@ mc_prep_clean <- function(data, silent=FALSE, resolve_conflicts=TRUE) {
 .prep_get_uncleaned_loggers <- function(data) {
     locality_function <- function(locality) {
         unprocessed <- purrr::discard(locality$loggers, .prep_is_logger_cleaned)
-        purrr::map_chr(unprocessed, function(x) x$metadata@serial_number)
+        if(length(unprocessed) == 0) {
+            return(tibble::tibble())
+        }
+        return(tibble::tibble(locality_id=locality$metadata@locality_id,
+                              logger_name=names(unprocessed)))
     }
-    loggers <- purrr::map(data$localities, locality_function)
-    purrr::reduce(loggers, c)
+    result <- purrr::map_dfr(data$localities, locality_function)
+    return(result)
 }
 
 .prep_check_datetime_step_unprocessed <- function(data, func=warning) {
@@ -296,7 +326,7 @@ mc_prep_clean <- function(data, silent=FALSE, resolve_conflicts=TRUE) {
 
 .prep_is_datetime_step_processed_in_object <- function(data) {
     unprocessed_loggers <- .prep_get_uncleaned_loggers(data)
-    return(length(unprocessed_loggers) == 0)
+    return(nrow(unprocessed_loggers) == 0)
 }
 
 .prep_print_clean_success_info <- function(data) {
@@ -309,7 +339,7 @@ mc_prep_clean <- function(data, silent=FALSE, resolve_conflicts=TRUE) {
     step_repr_function <- function(step) {
         return(stringr::str_glue("({step}s = {round(step/60, 2)}min)"))
     }
-    steps <- paste(purrr::map(sort(unique(info_table$step)), step_repr_function), collapse = ", ")
+    steps <- paste(purrr::map(sort(unique(info_table$step_seconds)), step_repr_function), collapse = ", ")
     message(stringr::str_glue("detected steps: {steps}"))
     print.data.frame(info_table)
 }
@@ -536,34 +566,42 @@ mc_prep_solar_tz <- function(data) {
 
 #' Crop datetime
 #'
-#' This function crop data by datetime
+#' This function crops data by datetime
 #'
 #' @details
 #' Function is able to crop data from `start` to `end` but works also
-#' with `start` only and `end` only. When only `start` is provided, then function crops only
-#' the beginning of the tim-series and vice versa with end.
+#' with `start` only or `end` only. When only `start` is provided, then function crops only
+#' the beginning of the time-series and vice versa using `end`.
 #'
-#' If `start` or `end` is a single POSIXct value, it is used for all or selected localities (regular crop).
-#' However, if `start` and `end` are vectors of POSIXct values with the same length as the localities vector,
-#' each locality is cropped by its own time window (irregular crop).
+#' For advanced cropping per individual locality and logger use `crop_table` parameter. 
+#' Crop_table is r data.frame containing columns:
+#' * `locality_id` - e.g. Loc_A1
+#' * `logger_name` - e.g. TMS_1 see [mc_info_logger] 
+#' * `start` - POSIXct datetime in UTC
+#' * `end` - POSIXct datetime in UTC
+#' 
+#' If `logger_name` is NA, then all loggers at certain locality are cropped. The column `logger_name`
+#' is ignored in agg-format. The `start` or `end` can be NA, then the data are not cropped.
+#' If the `crop_table` is provided, then `start`, `end` and `localities` parameters must be NULL.
 #'
-#' The `end_included` parameter is used for selecting, whether to return data which contains `end`
+#' The `end_included` parameter is used for specification, whether to return data which contains `end`
 #' time or not. For example when cropping the data to rounded days, typically users use midnight.
 #' 2023-06-15 00:00:00 UTC. But midnight is the last date of ending day and the same
-#' time first date of the next day. Thus, there will be the last day with single record.
-#' This can be confusing in aggregation (e.g. daily mean of single record per day, typically NA) so
-#' sometimes it is better to exclude end and crop on 2023-06-14 23:45:00 UTC (15 minutes records).
+#' time first date of the next day. This will create the last day of time-series containing single record (midnight).
+#' This can be confusing when user performs aggregation with such data (e.g. daily mean of single record per day, typically NA) so
+#' sometimes it is better to use `end_included =  FALSE` excluding end record and crop at 2023-06-14 23:45:00 UTC (15 minutes records).
 #'
 #' @template param_myClim_object
-#' @param start optional; POSIXct datetime **in UTC**; single value or vector; start datetime is included (default NULL)
-#' @param end optional, POSIXct datetime **in UTC**; single value or vector (default NULL)
+#' @param start optional; POSIXct datetime **in UTC** value; start datetime is included (default NULL)
+#' @param end optional; POSIXct datetime **in UTC** value (default NULL)
 #' @param localities vector of locality_ids to be cropped; if NULL then all localities are cropped (default NULL)
 #' @param end_included if TRUE then end datetime is included (default TRUE), see details
+#' @param crop_table data.frame (table) for advanced cropping; see details
 #' @return cropped data in the same myClim format as input.
 #' @export
 #' @examples
 #' cropped_data <- mc_prep_crop(mc_data_example_clean, end=as.POSIXct("2020-02-01", tz="UTC"))
-mc_prep_crop <- function(data, start=NULL, end=NULL, localities=NULL, end_included=TRUE) {
+mc_prep_crop <- function(data, start=NULL, end=NULL, localities=NULL, end_included=TRUE, crop_table=NULL) {
     if(!is.null(start) && any(format(start, format="%Z") != "UTC")) {
         warning(stringr::str_glue("start datetime is not in UTC"))
     }
@@ -574,53 +612,84 @@ mc_prep_crop <- function(data, start=NULL, end=NULL, localities=NULL, end_includ
         !.prep_crop_is_datetime_correct(end, localities)) {
         stop(.prep_const_MESSAGE_CROP_DATETIME_LENGTH)
     }
+    .prep_check_parameters_for_crop(start, end, localities, crop_table)
     all_table <- tibble::tibble(locality_id=names(data$localities))
     if(!is.null(localities)) {
         table <- tibble::tibble(locality_id=localities)
-        table$start_datetime <- if(is.null(start)) lubridate::NA_POSIXct_ else start
-        table$end_datetime <- if(is.null(end)) lubridate::NA_POSIXct_ else end
+        table$start <- if(is.null(start)) lubridate::NA_POSIXct_ else start
+        table$end <- if(is.null(end)) lubridate::NA_POSIXct_ else end
         all_table <- dplyr::left_join(all_table, table, by="locality_id")
+    } else if(!is.null(crop_table)) {
+        all_table <- dplyr::left_join(all_table, crop_table, by="locality_id")
     }
     else {
-        all_table$start_datetime <- if(is.null(start)) lubridate::NA_POSIXct_ else start
-        all_table$end_datetime <- if(is.null(end)) lubridate::NA_POSIXct_ else end
+        all_table$start <- if(is.null(start)) lubridate::NA_POSIXct_ else start
+        all_table$end <- if(is.null(end)) lubridate::NA_POSIXct_ else end
     }
-
-    sensors_item_function <- function(item, start_datetime, end_datetime) {
-        .prep_crop_data(item, start_datetime, end_datetime, end_included)
+    if(!("logger_name" %in% colnames(all_table))){
+        all_table$logger_name <- NA_character_
     }
+    all_table <- dplyr::group_by(all_table, .data$locality_id)
 
-    raw_locality_function <- function(locality_id, start_datetime, end_datetime) {
-        locality <- data$localities[[locality_id]]
-        if(!is.na(start_datetime) || !is.na(end_datetime)) {
-            locality$loggers <- purrr::pmap(list(item=locality$loggers,
-                                                 start_datetime=start_datetime,
-                                                 end_datetime=end_datetime),
-                                            sensors_item_function)
+    crop_bar <- progress::progress_bar$new(format = "crop [:bar] :current/:total localities",
+                                           total=dplyr::n_groups(all_table))
+    crop_bar$tick(0)
+
+    sensors_item_function <- function(item, start, end) {
+        if(!is.na(start) || !is.na(end)) {
+            return(.prep_crop_data(item, start, end, end_included))
+        } else {
+            return(item)
         }
+    }
+
+    raw_locality_function <- function(locality_table, key_table) {
+        locality_id <- key_table$locality_id[[1]]
+        locality <- data$localities[[locality_id]]
+        if(anyDuplicated(locality_table$logger_name) > 0 ||
+           (nrow(locality_table) > 1 && any(is.na(locality_table$logger_name)))) {
+            stop(.prep_const_MESSAGE_CROP_TABLE_DUPLICATES)
+        }
+
+        logger_table <- data.frame(logger_name = names(locality$loggers))
+        if(is.na(locality_table$logger_name[[1]])) {
+            logger_table$start <- locality_table$start[[1]]
+            logger_table$end <- locality_table$end[[1]]
+        } else {
+            logger_table <- dplyr::left_join(logger_table, locality_table, by="logger_name")
+        }
+        
+        locality$loggers <- purrr::pmap(list(item=locality$loggers,
+                                             start=logger_table$start,
+                                             end=logger_table$end), sensors_item_function)
+        crop_bar$tick()
         return(locality)
     }
 
-    agg_locality_function <- function(locality_id, start_datetime, end_datetime) {
+    agg_locality_function <- function(locality_table, key_table) {
+        locality_id <- key_table$locality_id[[1]]
         locality <- data$localities[[locality_id]]
-        if(!is.na(start_datetime) || !is.na(end_datetime)) {
-            locality <- sensors_item_function(locality, start_datetime, end_datetime)
+        if(nrow(locality_table) > 1) {
+            stop(.prep_const_MESSAGE_CROP_TABLE_DUPLICATES)
         }
+        start <- locality_table$start[[1]]
+        end <- locality_table$end[[1]]
+        locality <- sensors_item_function(locality, start, end)
+        crop_bar$tick()
         return(locality)
     }
 
     if(.common_is_agg_format(data)) {
-        data$localities <- purrr::pmap(all_table, agg_locality_function)
+        data$localities <- dplyr::group_map(all_table, agg_locality_function)
     } else {
-        data$localities <- purrr::pmap(all_table, raw_locality_function)
+        data$localities <- dplyr::group_map(all_table, raw_locality_function)
     }
-    names(data$localities) <- all_table$locality_id
+    names(data$localities) <- purrr::map_chr(data$localities, ~ .x$metadata@locality_id)
     return(data)
 }
 
 .prep_crop_is_datetime_correct <- function(datetime, localities) {
-    return(is.null(datetime) || length(datetime) == 1 ||
-        (!is.null(localities) && length(datetime) == length(localities)))
+    return(is.null(datetime) || length(datetime) == 1)
 }
 
 .prep_crop_data <- function(item, start, end, end_included) {
@@ -672,6 +741,12 @@ mc_prep_crop <- function(data, start=NULL, end=NULL, localities=NULL, end_includ
     item
 }
 
+.prep_check_parameters_for_crop <- function(start, end, localities, crop_table) {
+    if(!is.null(crop_table) && (!is.null(start) || !is.null(end) || !is.null(localities))) {
+        stop(.prep_const_MESSAGE_CROP_TABLE_PARAMS)
+    }
+}
+
 #' Merge myClim objects
 #'
 #' @description
@@ -708,9 +783,9 @@ mc_prep_merge <- function(data_items) {
         locality1 <- data1$localities[[locality_id]]
         locality2 <- data2$localities[[locality_id]]
         if(is_raw_format) {
-            return(.prep_merge_prep_localities(locality1, locality2))
+            return(.prep_merge_raw_localities(locality1, locality2))
         }
-        .prep_merge_calc_localities(locality1, locality2, data1$metadata@period)
+        .prep_merge_agg_localities(locality1, locality2, data1$metadata@period)
     }
 
     common_localities <- purrr::map(common_locality_ids, merge_localities_function)
@@ -736,12 +811,13 @@ mc_prep_merge <- function(data_items) {
     }
 }
 
-.prep_merge_prep_localities <- function(locality1, locality2){
+.prep_merge_raw_localities <- function(locality1, locality2){
     locality1$loggers <- c(locality1$loggers, locality2$loggers)
+    locality1 <- .read_generate_logger_names(locality1)
     locality1
 }
 
-.prep_merge_calc_localities <- function(locality1, locality2, period){
+.prep_merge_agg_localities <- function(locality1, locality2, period){
     localities <- list(locality1, locality2)
     datetime <- .agg_get_datetimes_from_sensor_items(localities, period)
     sensors <- .agg_get_merged_sensors(datetime, localities)
@@ -781,7 +857,7 @@ mc_prep_merge <- function(data_items) {
 #' it is not allowed to provide new calibration data, neither run calibration again.
 #'
 #' @template param_myClim_object_raw
-#' @param calib_table data.frame with columns (serial_number, sensor_id, datetime, slope, intercept)
+#' @param calib_table data.frame with columns (`serial_number`, `sensor_id`, `datetime`, `cor_factor`, `cor_slope`)
 #' @return myClim object with loaded calibration information in metadata.
 #' Microclimatic records are not calibrated, only ready for calibration.
 #' To calibrate records run [myClim::mc_prep_calib()]
@@ -1086,13 +1162,13 @@ mc_prep_TMSoffsoil <- function(data,
     sdt12 <- t1_sd / t2_sd
     moist <- item$sensors[[moist_sensor]]$values
     minmoist  <- .prep_apply_function_to_window(moist, count_values_per_day + 1, min, na.rm = TRUE)
-    result_values <- ifelse(sdt12 < sd_threshold, 0, ifelse(minmoist >= minmoist_threshold, 0, 1))
+    result_values <- ifelse(sdt12 < sd_threshold & !is.na(sdt12), 0, ifelse(minmoist >= minmoist_threshold, 0, 1))
     if(smooth) {
         result_values <- .prep_smoothing_rolling_mean(result_values, smooth_window * count_values_per_day + 1,
                                                       threshold = smooth_threshold, na.rm = TRUE)
     }
     item$sensors[[output_sensor]] <- .common_get_new_sensor(mc_const_SENSOR_logical, output_sensor,
-                                                            values=result_values)
+                                                            values=as.logical(result_values))
     return(item)
 }
 
