@@ -8,11 +8,15 @@
 .read_const_MESSAGE_UNSUPPOERTED_FORMAT <- "{data_format} is not a supported data format. File is skipped."
 .read_const_MESSAGE_UNAPLICABLE_FORMAT <- "{data_format} is not applicable format to {path}. File is skipped."
 .read_const_MESSAGE_USER_DATA_FORMAT_KEY <- "The key in user_data_format must not be the same as the key in mc_data_formats."
-.read_const_MESSAGE_VROOM_WARNING <- "Parsing issues in file {filename}"
+.read_const_MESSAGE_VROOM_WARNING <- "Parsing issues in file {filename}. Check mc_read_problems[['{filename}']]."
 .read_const_MESSAGE_FILE_SKIP <- "File {.x} does not exist - skipping."
 
 .read_state <- new.env()
 
+#' Environment for reading problems
+#' @export
+mc_read_problems <- new.env()
+
 #' Reading files or directories
 #'
 #' This function read one or more CSV/TXT files or directories of identical, 
@@ -250,7 +254,7 @@ mc_read_data <- function(files_table, localities_table=NULL, clean=TRUE, silent=
 }
 
 .read_parse_date_format <- function(files_table) {
-    if(is.null(files_table$date_format)) {
+    if(!("date_format" %in% colnames(files_table))) {
         return(files_table)
     }
     condition <- purrr::map_lgl(files_table$date_format, ~ all(!is.na(.x)) && length(.x) == 1 && is.character(.x)) 
@@ -385,7 +389,8 @@ mc_read_data <- function(files_table, localities_table=NULL, clean=TRUE, silent=
                            step = step)
         locality$loggers <- purrr::pmap(parameters, .read_logger)
         locality$loggers <- purrr::discard(locality$loggers, ~ is.null(.x))
-        locality
+        locality <- .read_generate_logger_names(locality)
+        return(locality)
     }
     result_localities <- dplyr::group_map(groupped_files, locality_function)
     result_localities <- purrr::discard(result_localities, ~ length(.x$loggers) == 0)
@@ -409,6 +414,32 @@ mc_read_data <- function(files_table, localities_table=NULL, clean=TRUE, silent=
     list(metadata = metadata, loggers=list())
 }
 
+.read_generate_logger_names <- function(locality) {
+    logger_names <- .read_get_logger_names(locality)
+    locality$loggers <- purrr::map2(locality$loggers, logger_names, function (logger, name) {
+        logger$metadata@name <- name
+        return(logger)})
+    names(locality$loggers) <- logger_names
+    return(locality)
+}
+
+.read_get_logger_names <- function(locality) {
+    logger_types <- purrr::map_chr(locality$loggers, ~ .x$metadata@type)
+    logger_types[is.na(logger_types)] <- .model_const_LOGGER_NA_TYPE_NAME
+    env_index <- new.env()
+    index_function <- function(logger_type) {
+        if(is.null(env_index[[logger_type]])) {
+            env_index[[logger_type]] <- 1
+        } else {
+            env_index[[logger_type]] <- env_index[[logger_type]] + 1
+        }
+        return(env_index[[logger_type]])
+    }
+    logger_ids <- purrr::map_int(logger_types, index_function)
+    logger_names <- purrr::map2_chr(logger_types, logger_ids, ~ stringr::str_glue("{.x}_{.y}"))
+    return(logger_names)
+}
+
 .read_get_data_raw_from_localities <- function(localities) {
     metadata <- new("mc_MainMetadata")
     myClimList(metadata, localities)
@@ -454,6 +485,7 @@ mc_read_data <- function(files_table, localities_table=NULL, clean=TRUE, silent=
         problems <- vroom::problems(result)
     }
     if(nrow(problems) > 0) {
+        mc_read_problems[[filename]] <- problems
         warning(stringr::str_glue(.read_const_MESSAGE_VROOM_WARNING))
     }
     return(result)
@@ -596,7 +628,8 @@ mc_read_wide <- function(data_table, sensor_id=mc_const_SENSOR_real, sensor_name
         sensors <- list()
         sensors[[sensor_name]] <- .common_get_new_sensor(sensor_id, sensor_name, data_table[[locality$metadata@locality_id]])
         locality$loggers[[1]] <- .read_get_new_logger(data_table[[1]], sensors)
-        locality
+        locality <- .read_generate_logger_names(locality)
+        return(locality)
     }
     localities <- purrr::map(result, locality_function)
     data <- .read_get_data_raw_from_localities(localities)
@@ -676,7 +709,8 @@ mc_read_long <- function(data_table, sensor_ids=list(), clean=TRUE, silent=FALSE
     sensors <- purrr::map(sensor_names, sensor_function)
     names(sensors) <- purrr::map_chr(sensors, ~ .x$metadata@name)
     result$loggers[[1]] <- .read_get_new_logger(table_values$datetime, sensors)
-    result
+    result <- .read_generate_logger_names(result)
+    return(result)
 }
 
 #' Reading data from TubeDB
